/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */
/**
 * <BODY>
 * <P>
 * A general-purpose API for ontologies.
 * These are directed graphs consisting of <code>Term</code> obects.  Each
 * edge in the graph is annotated with a term which defines the relationship
 * between its subject and object terms.  In this API, these annotated
 * edges are called triples.
 * </P>
 * <P>Some interfaces and implementations from this package are extended in
 * {@link org.biojavax.ontology biojavax} to better facilitate their persistence to
 * biosql.</P>
 * </BODY>
 */