/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */
package org.biojava.nbio.proteincomparisontool;

/**
 * A dummy class that does nothing. This is so we can upload protein-comparison-tool.jar to Maven Central
 *
 */
public class HelloWorld {

	public HelloWorld(){
		// hello world
	}

	/**
	 * Prints hello world on command line
	 *
	 */
	public void print(){
		System.out.println("Hello World!");
	}
}
