package org.biojava.nbio.structure.io.cif;

import java.io.Serializable;

/**
 * Flag for BioJava beans that resemble categories defined by the mmCIF schema.
 * @author Sebastian Bittrich
 * @since 6.0.0
 */
public interface CifBean extends Serializable {
}
