/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */
package org.biojava.nbio.structure.align.util;


public class ConfigurationException extends Exception {

	/**
	 *
	 */
	private static final long serialVersionUID = -8047100079715000276L;

	/**
	 * Constructs a ConfigurationException object.
	 *
	 * @param s  a String ...
	 */
	public ConfigurationException(String s) {
	super(s);
	}

	public ConfigurationException(String message, Throwable cause) {
		super(message, cause);
	}

	/**
	 * Constructs a ConfigurationException object.
	 *
	 * @param t  a Throwable object
	 */
	public ConfigurationException (Throwable t) {
	super(t);
	}

	public ConfigurationException() {
	}
}

